function make_diffusion

% input - regularized data for make_diffusion
global Rr Mnr Mgr Fer Car Pr Tr tr
global tr_rem Pr_rem Tr_rem 
global time_fac diff_mode BC B_conc
global Pi Ti ti
global Mni Mgi Fei Cai GVi
global iq_JJ

% output
clear cMn cMg cFe cCa 
global cMn cMg cFe cCa % actual and then final concentrations 

% units
% time tr in My (also ti)
% input temperature Tr in K, pressure Pr in bars (but in com_D T is in C)
% radius Rr in micrometers

% boundary conditions
% BCond_creation=0 % for the part of PT-path where garnet is created
BCond_creation=3; % for the part of PT-path where garnet is created

BCond_rem_part= BC % for the remaining part of PT-path (global)

% working variables (to save the originals)
Mnw=Mnr; Mgw=Mgr; Few=Fer; Caw=Car; 
trw=tr*time_fac;

ieach=1 % ieach=10 means each 10th profile will be drawn

hf1=figure('name','Diffusion path');
pos=get(gcf,'position');
set(gcf,'position',[10 pos(2) pos(3) pos(4)])
hold on, grid on 
xlabel('t (My)'), ylabel('T (K)'), zlabel('P (bar)')
plot3(ti*time_fac,Ti,Pi,'b.-') % *)
view(-35,30)
view(0,90)

hf2=figure('name','Diffusion profiles');
set(gcf,'position',[pos(1)+100 pos(2) pos(3) pos(4)])
hold on, grid on, xlabel('r'), ylabel('c'), box on

hf3=figure('name','Diffusion summary');
set(gcf,'position',[pos(1)+100 pos(2) pos(3) pos(4)])
hold on, grid on, xlabel('r'), ylabel('c'), box on
ind=1:length(Rr);
h1=plot(Rr(ind),Mnw(ind),'b.-');
h2=plot(Rr(ind),Mgw(ind),'r.-');
h3=plot(Rr(ind),Few(ind),'m.-');
h4=plot(Rr(ind),Caw(ind),'g.-');

figure(hf2)

% diffusion in the gradually growing garnet
iqsph=1; % spherical mode
nr=length(trw)
i0=10 % from i0 the diffusion begins
Tri(1:i0-1)=Tr(1:i0-1);
for i=i0:nr % go along the PT-path and spend time timint at each point
    % ti(i) ... actual time (My) at the end of garnet stay in i-th point
    if i==i0; timint=[trw(1) trw(i0)]; else; timint=[trw(i-1) trw(i)]; end

    figure(hf1)
    title(['Garnet at ',num2str(trw(i),3),' My'])
    plot3(tr(i)*time_fac,Tr(i),Pr(i),'ro')
%     if tr(i)>0.2, pause, end
    
    figure(hf2)
    title(['Mn(b) Mg(r) Fe(m) Ca(g)',' at ',num2str(trw(i),3),' My'])
    
    ind=[1:i];
    
%     if mod(i,ieach)==0
%         h1=plot(Rr(ind),Mnw(ind),'b.-');
%         h2=plot(Rr(ind),Mgw(ind),'r.-');
%         h3=plot(Rr(ind),Few(ind),'m.-');
%         h4=plot(Rr(ind),Caw(ind),'g.-');
%         pause(0.1)
%         delete(h1),delete(h2),delete(h3),delete(h4)
%     end

    Ta=Tr(i)-273; Pa=Pr(i); % Ta enters in C 
    [cMn,cMg,cFe,cCa,De]=diff_step(Mnw(ind),Mgw(ind),Few(ind),Caw(ind),...
        Rr(ind),Pa,Ta,diff_mode,iqsph,timint,BCond_creation,B_conc,...
        Pi,Ti,ti,time_fac,Mni,Mgi,Fei,Cai,GVi,hf1,hf2);
    Mnw(ind)=cMn; Mgw(ind)=cMg; Few(ind)=cFe; Caw(ind)=cCa;
    
    Tri(i)=Tr(i); Dei(i)=De; % for further use ***)
    
    figure(hf2)

    if mod(i,ieach)==0
%         delete(h1),delete(h2),delete(h3),delete(h4)       
        hh1=plot(Rr(ind),cMn,'b-');
        hh2=plot(Rr(ind),cMg,'r-');
        hh3=plot(Rr(ind),cFe,'m-');
        hh4=plot(Rr(ind),cCa,'g-');
    end
    pause(0.001)

%     if tr(i)>0.05, tri=tr(i), 'pause', pause, end
end

figure(hf2)
h1=plot(Rr(ind),cMn,'b-');
h2=plot(Rr(ind),cMg,'r-');
h3=plot(Rr(ind),cFe,'m-');
h4=plot(Rr(ind),cCa,'g-');
h1=plot(Rr(ind),cMn,'k:');
h2=plot(Rr(ind),cMg,'k:');
h3=plot(Rr(ind),cFe,'k:');
h4=plot(Rr(ind),cCa,'k:');

figure(hf3)
h1=plot(Rr(ind),cMn,'k-');
h2=plot(Rr(ind),cMg,'k-');
h3=plot(Rr(ind),cFe,'k-');
h4=plot(Rr(ind),cCa,'k-');

cMnbc=cMn; cMgbc=cMg; cFebc=cFe; cCabc=cCa; % for iq_JJ only

figure(hf2)

'diffusion in growing garnet completed'
'diffusion will continue along the remaining PT-path'
% pause, 'press enter'
pause(.001)
% pause

% diffusion along the remaining PT-path
tr_rem
if tr_rem~=0
tr_remw=tr_rem*time_fac;
ntri=length(Tri);
nri=length(tr_remw);
    for i=1:nri
        
        if i==1; timint=[trw(end) tr_remw(1)]; else; timint=[tr_remw(i-1) tr_remw(i)]; end
        if timint==0, 'timint==0', pause, end
        
        figure(hf1)
        title(['Garnet path at ',num2str(tr_remw(i),3),' My'])
        plot3(tr_rem(i)*time_fac,Tr_rem(i),Pr_rem(i),'g.')
        
        figure(hf2)
        title(['Mn(b) Mg(r) Fe(m) Ca(g)',' after ',num2str(tr_remw(i),3),' My'])
               
        Ta=Tr_rem(i)-273; Pa=Pr_rem(i); % Ta enters in C
        [cMn,cMg,cFe,cCa,De]=diff_step(Mnw,Mgw,Few,Caw,Rr,Pa,Ta,diff_mode,...
            iqsph,timint,BCond_rem_part,B_conc,...
            Pi,Ti,ti,time_fac,Mni,Mgi,Fei,Cai,GVi,hf1,hf2);
        Mnw=cMn; Mgw=cMg; Few=cFe; Caw=cCa;

        Tri(ntri+i)=Tr_rem(i); Dei(ntri+i)=De; % for further use ***)
        
        figure(hf2)

        if mod(i,ieach)==0
            %         delete(h1),delete(h2),delete(h3),delete(h4)
            h1=plot(Rr,cMn,'b-');
            h2=plot(Rr,cMg,'r-');
            h3=plot(Rr,cFe,'m-');
            h4=plot(Rr,cCa,'g-');
            pause(0.001)
        end
        pause(0.001)
        
    end
end

figure(hf2)
h1=plot(Rr,cMn,'b-','linewidth',2);
h2=plot(Rr,cMg,'r-','linewidth',2);
h3=plot(Rr,cFe,'m-','linewidth',2);
h4=plot(Rr,cCa,'g-','linewidth',2);
h1=plot(Rr,cMn,'k-');
h2=plot(Rr,cMg,'k-');
h3=plot(Rr,cFe,'k-');
h4=plot(Rr,cCa,'k-');

figure(hf3)
title(['Summary: without diffusion (.), end of garnet formation (thin-), final (thick-)'],...
    'fontsize',9)
h1=plot(Rr,cMn,'b-','linewidth',2);
h2=plot(Rr,cMg,'r-','linewidth',2);
h3=plot(Rr,cFe,'m-','linewidth',2);
h4=plot(Rr,cCa,'g-','linewidth',2);

figure(hf2)
BCond_creation
BCond_rem_part

if iq_JJ==1 % for JJ only --------------------------------------------
% pozor na to, ze eval difusivity muze byt komplexni            
% if isreal(De)==0
% De, 'De complex, imaginary part omitted', De=real(De), pause
% end
% asi by stalo za to si vypsat jednotlive diffs a vykreslit
% asi bychom se meli zabyvat nastaveni Ca x uphill diff
% Vielzeuf vychazi dost odlisne (Ch mu moc neveri)

%     figure, hold on %***)
%     title('diffusivity along the path')
%     plot(Tri,Dei,'b.-')
%     length(Tri)
%     length(tr)
%     length(tr_rem)
%     figure, hold on %***)
%     title('diffusivity along the path')
%     plot([tr tr_rem],Dei,'b.-')
    figure, hold on %***)
    title('maximum diffusivity along the path')
    plot3([tr tr_rem],Tri,Dei,'b.-')
    plot3([tr tr_rem],[Tr Tr_rem],Dei,'r.')
    grid on
    xlabel('time')
    ylabel('T(K)')
    zlabel('diffusivity')
    zlabel('D')
    view(-35,30)
    
    
    hjj=figure,; hold on
    h1=plot(Rr(ind).^3,cMnbc,'k-');
    h2=plot(Rr(ind).^3,cMgbc,'k-');
    h3=plot(Rr(ind).^3,cFebc,'k-');
    h4=plot(Rr(ind).^3,cCabc,'k-');
    %     figure(hjj), hold on
    title(['Summary: porovnej plosky :)'],...
        'fontsize',9)
    xlabel('r^3')
    h1=plot(Rr.^3,cMn,'b-','linewidth',1);
    h2=plot(Rr.^3,cMg,'r-','linewidth',1);
    h3=plot(Rr.^3,cFe,'m-','linewidth',1);
    h4=plot(Rr.^3,cCa,'g-','linewidth',1);
    
    Fe=cFebc-cFe;
    Mg=cMgbc-cMg;
    Ca=cCabc-cCa;
    sumFe=sum(Fe), sum(Fe(2:end))
    sumMg=sum(Mg), sum(Mg(2:end))
    sumCa=sum(Mg), sum(Ca(2:end))
    meanFe=mean(Fe),     meanMg=mean(Mg),     meanCa=mean(Ca)
%     plot(Rr.^3,Fe*10,'m--')
%     plot(Rr.^3,Mg*10,'r--')
%     plot(Rr.^3,Ca*10,'g--')
end

% -------------------------------------------------------------------------

function [cMn,cMg,cFe,cCa,De]=diff_step(cMn,cMg,cFe,cCa,r,...
    Pa,Ta,diff_mode,iqsph,timint,BCond,B_conc,...
    Pi,Ti,ti,time_fac,Mni,Mgi,Fei,Cai,GVi,hf1,hf2);

% Ta is in C, Pa in bars, r in micrometers (D=microm^2/s)

if diff(timint)==0
    % 'zero time interval - no diffusion'
    De=NaN
    return
end

            tif=ti*time_fac;

ff=1; % fugacity ratio

secMy=3600*24*365*1000000;
dtti=(ti(2)-ti(1))*secMy

nc=length(r);
check_nc=length(cMn);
if nc~=check_nc; 'nc~=check_nc', pause, end

dx=r(2)-r(1);
'---'
timint
% delt_My=diff(timint)
t=timint*secMy;
nt=length(t); % = 2

for i=2:nc-1 % nodes for spherical model
    im=i-1; ip=i+1;
    r2i(i)=(i*dx)^2; r2mi(i)=((i-1/2)*dx)^2; r2pi(i)=((i+1/2)*dx)^2;
end

iqwrite=0; % output D for testing 
% nc2=fix(nc/2)
for it=1:nt-1 % (it=1 but I let the loop here for further use)
    
    tim=t(it);
    tim_My=tim/secMy
    while tim<t(it+1)
        
        % find optimal time step dt (with a correction by the end)
        % diffusivities
        [DD,DMn,DMg,DFe,DCa]=com_D(Pa,Ta,cMn(1),cMg(1),cFe(1),cCa(1),ff,diff_mode);
        if iqwrite==1
            'diffusivities in the centre (microm^2/s)', DMn,DMg,DFe,DCa
            % iqwrite=0;
        end   

        [eve,eva]=eig(DD); De1=max(max(abs(eva)));
        % [DD,DMn,DMg,DFe,DCa]=com_D(Pa,Ta,cMn(nc2),cMg(nc2),cFe(nc2),cCa(nc2),ff,diff_mode);
        % [eve,eva]=eig(DD); De2=max(max(abs(eva)));
        [DD,DMn,DMg,DFe,DCa]=com_D(Pa,Ta,cMn(end),cMg(end),cFe(end),cCa(end),ff,diff_mode);
        if iqwrite==1
            'diffusivities at the rim (microm^2/s)', DMn,DMg,DFe,DCa
            % iqwrite=0;
        end
        [eve,eva]=eig(DD); De3=max(max(abs(eva)));
        % De=max([De1,De2,De3]);
        De=max([De1,De3]);
        % dtimemax=1/2*dx^2/D
        % dt=0.1*dx^2/De;
        dt=0.3*dx^2/De;
        dt0=dt/secMy % jen vypis
        % dt=min(dt,t(it+1)-tim)
        dt=min([dt,t(it+1)-tim,dtti])
        QQ=dt/dx^2;
        if dt==0, 'diff_step, dt==0', pause, end
        tim1=tim+dt;
        if tim1<=t(it+1); tim=tim1; else; dt=dt-(tim1-t(it+1)); tim=t(it+1); end
        dt1=dt/secMy % jen vypis
        tim_My=tim1/secMy % now the actual time is tim1
        
        % Ta ,De, dt_My=dt/secMy     
        Pa=interp1(tif,Pi,tim_My); Ta=interp1(tif,Ti,tim_My)-273; % actual PT
 
        % evaluate concentrations
        C=[cMn',cMg',cFe']; 
        % BC
        % garnet center
        C(1,:)=C(2,:); % ~gradC=0
        % garnet rim
        if BCond==0
            %
        elseif BCond==1
            Mnw_end=B_conc(1);
            Mgw_end=B_conc(2);
            Few_end=B_conc(3);
            Caw_end=1-Mnw_end-Mgw_end-Few_end;
            C(nc,:)=[Mnw_end,Mgw_end,Few_end];         
        elseif BCond==2
            C(nc,:)=C(nc-1,:); % ~gradC=0
        elseif BCond==3
            % ti je nyni patrne transformovan (timefac ci make_time)
            % je treba zajistit, aby ti byl spravne           
            %  prirazeni do posledniho radku C=[cMn',cMg',cFe'];
            Few_end=interp1(tif,Fei,tim_My);
            Mnw_end=interp1(tif,Mni,tim_My);
            Mgw_end=interp1(tif,Mgi,tim_My);
            Caw_end=interp1(tif,Cai,tim_My);
            suma=Few_end+Mnw_end+Mgw_end+Caw_end;
            Few_end=Few_end/suma;
            Mgw_end=Mgw_end/suma;
            Mnw_end=Mnw_end/suma;
            % Caw(end)=1-Mnw(end)-Mgw(end)-Few(end);
            C(nc,:)=[Mnw_end,Mgw_end,Few_end];         
        elseif BCond==4           
            %  Mnw_end=0, prirazeni do posledniho radku C=[cMn',cMg',cFe'];
            Few_end=interp1(tif,Fei,tim_My);
            Mnw_end=0;
            Mgw_end=interp1(tif,Mgi,tim_My);
            Caw_end=interp1(tif,Cai,tim_My);
            suma=Mgw_end + Few_end + Caw_end;
            Few_end=Few_end/suma;
            Mgw_end=Mgw_end/suma;
            Mnw_end=Mnw_end/suma;
            C(nc,:)=[Mnw_end,Mgw_end,Few_end];     
        end  
        
        figure(hf1)
        plot3(tim_My,Ta+273,Pa,'bx','markersize',5)

        CC=cCa';
        A(1,:)=C(1,:); A(nc,:)=C(nc,:);
        if iqsph==1 % spherical geometry
            for i=2:nc-1
                im=i-1; ip=i+1;
                Cw(1,:)=(C(i,:)+C(im,:))/2; CCw=(CC(i)+CC(im))/2;
                [Dmi(:,:,i),DMn,DMg,DFe,DCa]=com_D(Pa,Ta,Cw(1),Cw(2),Cw(3),CCw,ff,diff_mode);
                Cw(1,:)=(C(i,:)+C(ip,:))/2; CCw=(CC(i)+CC(ip))/2;
                [Dpi(:,:,i),DMn,DMg,DFe,DCa]=com_D(Pa,Ta,Cw(1),Cw(2),Cw(3),CCw,ff,diff_mode);
            end
            for i=2:nc-1
                im=i-1; ip=i+1;
                r2=r2i(i); r2m=r2mi(i); r2p=r2pi(i);
                Dm=Dmi(:,:,i); Dp=Dpi(:,:,i);
                A(i,:)=C(i,:)+QQ*(r2p*Dp*(C(ip,:)-C(i,:))'-r2m*Dm*(C(i,:)-C(im,:))')'/r2;
            end
        else % linear
            for i=1:nc
                [Di(:,:,i),DMn,DMg,DFe,DCa]=com_D(Pa,Ta,C(i,1),C(i,2),C(i,3),CC(i),ff,diff_mode);
            end
            for i=2:nc-1
                im=i-1; ip=i+1;
                D=Di(:,:,i); Dp=Di(:,:,ip); dD=Dp-D;
                A(i,:)=C(i,:)+QQ*(dD*(C(ip,:)-C(i,:))'+D*(C(ip,:)-2*C(i,:)+C(im,:))')';
            end
        end
        C=A; % CC(:)=1-C(:,1)-C(:,2)-C(:,3);
     

        cMn=C(:,1)'; cMg=C(:,2)'; cFe=C(:,3)';  % new concentrations
        CC(:)=1-C(:,1)-C(:,2)-C(:,3); cCa=CC'; 
        
    end % while tim<t(it+1)
       
    if min(cCa)<0; 'cCa<0', DD, eva, pause, end

    figure(hf2)
    hh1=plot(r,cMn,'b-');
    hh2=plot(r,cMg,'r-');
    hh3=plot(r,cFe,'m-');
    hh4=plot(r,cCa,'g-');
    '---------------------', pause(0.001)
    
end

% -------------------------------------------------------------------------

function [D,DMn,DMg,DFe,DCa]=com_D(p,T,cMn,cMg,cFe,cCa,ff,diff_mode)

% diff_mode = 1 ... Chakraborty and Ganguly 1992 with Ca=0.05Mn 
%                   (or can be set Ca=0.5Fe as in Vielzouf, or smaller)
%             2 ... Carlson 2006, Table 4,with personal communication
%                (needs concentrations)
%             3 ... Vielzeuf 2007, Table 5 (his Ca, others as ChG92)
%             <0 and >3 ... testing
% notes
% input temperature T in C, pressure p in bars, D microm^2/s
% large differences exist due to extrapolation of expereimental data
% see notes in the earlier versions of com_D (before 2019)
% R=1.9872 cal/K/mol, 1 cal=4.1841 J, R=8.3144621; J/K/mol

if diff_mode==-2 % for testing,  dif Mn a Mg = Fe, Ca = 100 x less
    DDFe=6.36e-4*1e8;
    DD3=DDFe*exp(-(65824+(p-1)*5.63/41.84)/(1.9872*(T+273)));
    DD1=DD3; DD2=DD3;
    % DD4=DD3;
    DD4=DD1*0.01;
    DMn=DD1; 
    DMg=DD2;
    DFe=DD3;
    DCa=DD4;
    
elseif diff_mode==-1 % for testing,  dif Mn a Mg = Fe, Ca = 20x
    DDFe=6.36e-4*1e8;
    DD3=DDFe*exp(-(65824+(p-1)*5.63/41.84)/(1.9872*(T+273)));
    DD1=DD3; DD2=DD3;
    % DD4=DD3;
    DD4=DD1*0.05;
    DMn=DD1;
    DMg=DD2;
    DFe=DD3;
    DCa=DD4;
    
elseif diff_mode==0 % for testing,  all = Fe
    DDFe=6.36e-4*1e8;
    DD3=DDFe*exp(-(65824+(p-1)*5.63/41.84)/(1.9872*(T+273)));
    DD1=DD3; DD2=DD3;
    DD4=DD3;
    DMn=DD1; 
    DMg=DD2;
    DFe=DD3;
    DCa=DD4;
    
elseif diff_mode==1 % Chakraborty and Ganguly  1992 (with a correction)
    DDMn=5.15e-4*1e8; 
    DDMg=1.11e-3*1e8;
    DDFe=6.36e-4*1e8;
    DD1=DDMn*exp(-(60569+(p-1)*6.04/41.84)/(1.9872*(T+273)));
    DD2=DDMg*exp(-(67997+(p-1)*5.27/41.84)/(1.9872*(T+273)));
    DD3=DDFe*exp(-(65824+(p-1)*5.63/41.84)/(1.9872*(T+273)));
    DD4=DD1*0.05;
    DMn=DD1;
    DMg=DD2;
    DFe=DD3;
    DCa=DD4;
    
elseif diff_mode==2 % Carlson 2006, Table 4 (in SI)
    % Table 4, lnD0 k Q dV (m2/s, 1/m, J/mol, m3/mol)
    TC=[...
        -17.638  465.7*1e9  264.55*1e3  13.538*1e-6; % Fe
        -19.947  419.9*1e9  244.21*1e3   8.565*1e-6; % Mg
        -17.619  496.9*1e9  264.44*1e3   9.631*1e-6; % Mn
        -22.572  511.1*1e9  230.56*1e3   9.795*1e-6]; % Ca
    a0alm=1.1525*1e-9; % m
    a0prp=1.1456*1e-9; % m
    a0sps=1.1614*1e-9; % m
    a0grs=1.1852*1e-9; % m
    Xalm=cFe; Xprp=cMg; Xsps=cMn;
    Xgrs=1-Xalm-Xprp-Xsps;
    a0=Xalm*a0alm+Xprp*a0prp+Xsps*a0sps+Xgrs*a0grs;
    aa=a0-a0alm; % alm is ref
    fac=1e12; % m2/s to mikro2/s
    pPas=p*1e5; % % pPas in Pascals (SI) (input bars)
    DFe=com_D_eq_table(TC(1,:),pPas,T,aa,ff)*fac; % microm^2/s
    DMg=com_D_eq_table(TC(2,:),pPas,T,aa,ff)*fac;
    DMn=com_D_eq_table(TC(3,:),pPas,T,aa,ff)*fac;
    DCa=com_D_eq_table(TC(4,:),pPas,T,aa,ff)*fac;  
    DD1=DMn; 
    DD2=DMg;
    DD3=DFe;
    DD4=DCa;
    
elseif diff_mode==3 % Vielzeuf 2007
    % Fe Mg Mn jako ChG 1992 (diff_mode=1), Ca as Vielzeuf
    % p in bars
    R=8.3144621; % J/K/mol
    DFe0=6.36e-4*1e8;    QFe=275408;    dVFe=0.563;
    DFe=DFe0*exp(-(QFe+(p-1)*dVFe)/R/(T+273)); 
    DMg0=1.11e-3*1e8;    QMg=284499;    dVMg=0.53;
    DMg=DMg0*exp(-(QMg+(p-1)*dVMg)/R/(T+273)); 
    DMn0=5.15e-4*1e8;    QMn=253421;    dVMn=0.60;
    DMn=DMn0*exp(-(QMn+(p-1)*dVMn)/R/(T+273)); 
    DCa0=6.63e-10*1e8;    QCa=179894;    dVCa=0.6;
    DCa=DCa0*exp(-(QCa+(p-1)*dVCa)/R/(T+273)); 
    DD1=DMn; % microm^2/s
    DD2=DMg;
    DD3=DFe;
    DD4=DCa;
    
elseif diff_mode==4 % test Borinski 2012
    R=8.3144621; % J/K/mol
    DFe0=1.64e-6*1e8;    QFe=226900;    dVFe=0.563;
    DFe=DFe0*exp(-(QFe+(p-1)*dVFe)/R/(T+273)); 
    DMg0=2.72e-6*1e8;    QMg=228300;    dVMg=0.53;
    DMg=DMg0*exp(-(QMg+(p-1)*dVMg)/R/(T+273)); 
    DMn0=5.15e-4*1e8;    QMn=253421;    dVMn=0.60;
    DMn=DMn0*exp(-(QMn+(p-1)*dVMn)/R/(T+273)); 
    DCa0=6.63e-10*1e8;    QCa=179894;    dVCa=0.6;
    DCa=DCa0*exp(-(QCa+(p-1)*dVCa)/R/(T+273));  
    DD1=DMn; % microm^2/s
    DD2=DMg;
    DD3=DFe;
    DD4=DCa;
    
end

% diffusivity matrix (see, eg., Borinski 2012, eq. 1, Caddick 2010, eq. 5)
Q=DD1*cMn+DD2*cMg+DD3*cFe+DD4*cCa;
D(1,1)=DD1-DD1*cMn/Q*(DD1-DD4);
D(1,2)=   -DD1*cMn/Q*(DD2-DD4);
D(1,3)=   -DD1*cMn/Q*(DD3-DD4);
D(2,1)=   -DD2*cMg/Q*(DD1-DD4);
D(2,2)=DD2-DD2*cMg/Q*(DD2-DD4);
D(2,3)=   -DD2*cMg/Q*(DD3-DD4);
D(3,1)=   -DD3*cFe/Q*(DD1-DD4);
D(3,2)=   -DD3*cFe/Q*(DD2-DD4);
D(3,3)=DD3-DD3*cFe/Q*(DD3-DD4);


function DD=com_D_eq_table(TC,p,T,aa,ff)
% Carlson 2006 eq. (3), input SI
R=8.3144621; % J/K/mol
lnD0=TC(1); k=TC(2); Q=TC(3); dV=TC(4);
lnD=lnD0+k*aa-(Q+p*dV)/R./(T+273)+1/6*log(ff);
DD=exp(lnD);

